@@ -35,6 +35,9 @@ static int       qtcWMMoveLastX=-1;
 static int       qtcWMMoveLastY=-1;
 static int       qtcWMMoveTimer=0;
 static GtkWidget *qtcWMMoveDragWidget=NULL;
+//! keep track of the last rejected button event to reject it again if passed to some parent widget
+/*! this spares some time (by not processing the same event twice), and prevents some bugs */
+ GdkEventButton *qtcWMMoveLastRejectedEvent=NULL;
 
 static void qtcWMMoveStopTimer()
 {
@@ -48,6 +51,7 @@ static void qtcWMMoveReset()
     qtcWMMoveLastX=-1;
     qtcWMMoveLastY=-1;
     qtcWMMoveDragWidget=NULL;
+    qtcWMMoveLastRejectedEvent=NULL;
     qtcWMMoveStopTimer();
 }
 
@@ -87,28 +91,48 @@ static void qtcWMMoveTrigger(GtkWidget *w, int x, int y)
 
 static gboolean qtcWMMoveWithinWidget(GtkWidget *widget, GdkEventButton *event)
 {
-    GdkWindow *window = gtk_widget_get_parent_window(widget);
+    // get top level widget
+    GtkWidget *topLevel=gtk_widget_get_toplevel(widget);;
+    GdkWindow *window = topLevel ? gtk_widget_get_window(topLevel) : NULL;
 
     if(window)
     {
-        GtkAllocation alloc=GTK_IS_NOTEBOOK(widget) ? qtcTabGetTabbarRect(GTK_NOTEBOOK(widget)) : qtcWidgetGetAllocation(widget);
-        int           nx=0,
-                      ny=0;
+        GtkAllocation allocation;
+        int           wx=0, wy=0, nx=0, ny=0;
 
-        // Need to get absolute co-ordinates...
+        // translate widget position to topLevel
+        gtk_widget_translate_coordinates(widget, topLevel, wx, wy, &wx, &wy);
+
+        // translate to absolute coordinates
         gdk_window_get_origin(window, &nx, &ny);
-        alloc.x+=nx;
-        alloc.y+=ny;
+        wx += nx;
+        wy += ny;
 
-        return alloc.x<=event->x_root && alloc.y<=event->y_root &&
-               (alloc.x+alloc.width)>event->x_root &&(alloc.y+alloc.height)>event->y_root;
+        // get widget size.
+        // for notebooks, only consider the tabbar rect
+        if(GTK_IS_NOTEBOOK(widget))
+        {
+            GtkAllocation widgetAlloc=qtcWidgetGetAllocation(widget);
+            allocation=qtcTabGetTabbarRect(GTK_NOTEBOOK(widget));
+            allocation.x += wx - widgetAlloc.x;
+            allocation.y += wy - widgetAlloc.y;
+        }
+        else
+        {
+            allocation = qtcWidgetGetAllocation(widget);
+            allocation.x = wx;
+            allocation.y = wy;
+        }
+
+        return allocation.x<=event->x_root && allocation.y<=event->y_root &&
+               (allocation.x+allocation.width)>event->x_root &&(allocation.y+allocation.height)>event->y_root;
     }
     return TRUE;
 }
 
-static gboolean isBlackListed(GObject *object)
+static gboolean qtcWMMoveIsBlackListed(GObject *object)
 {
-    static const char *widgets[]={ "GtkPizza", "GladeDesignLayout", "MetaFrames" , 0 };
+    static const char *widgets[]={ "GtkPizza", "GladeDesignLayout", "MetaFrames", "SPHRuler", "SPVRuler", 0 };
     
     int i;
     
@@ -132,7 +156,8 @@ static gboolean qtcWMMoveChildrenUseEvent(GtkWidget *widget, GdkEventButton *eve
         // cast child to GtkWidget
         if(GTK_IS_WIDGET(child->data))
         {
-            GtkWidget* childWidget=GTK_WIDGET(child->data);
+            GtkWidget *childWidget=GTK_WIDGET(child->data);
+            GdkWindow *window=NULL;
 
             // check widget state and type
             if(GTK_STATE_PRELIGHT==qtcWidgetGetState(childWidget))
@@ -140,45 +165,29 @@ static gboolean qtcWMMoveChildrenUseEvent(GtkWidget *widget, GdkEventButton *eve
                 // if widget is prelight, we don't need to check where event happen,
                 // any prelight widget indicate we can't do a move
                 usable = FALSE;
+                continue;
             }
-            else if(GTK_IS_NOTEBOOK( childWidget ) )
+
+            window = gtk_widget_get_window(childWidget);
+            if(!(window && gdk_window_is_visible(window)))
+                continue;
+
+            if(GTK_IS_NOTEBOOK(childWidget))
+                inNoteBook = true;
+
+            if(!(event && qtcWMMoveWithinWidget(childWidget, event)))
+                continue;
+
+            // check special cases for which grab should not be enabled
+            if(
+                (qtcWMMoveIsBlackListed(G_OBJECT(childWidget))) ||
+                (GTK_IS_NOTEBOOK(widget) && qtcTabIsLabel(GTK_NOTEBOOK(widget), childWidget)) ||
+                (GTK_IS_BUTTON(childWidget) && gtk_widget_get_state(childWidget) != GTK_STATE_INSENSITIVE) ||
+                (gtk_widget_get_events(childWidget) & (GDK_BUTTON_PRESS_MASK|GDK_BUTTON_RELEASE_MASK)) ||
+                (GTK_IS_MENU_ITEM(childWidget)) ||
+                (GTK_IS_SCROLLED_WINDOW(childWidget) && (!inNoteBook || gtk_widget_is_focus(childWidget))))
             {
-                inNoteBook = TRUE;
-            }
-            else if(event && qtcWMMoveWithinWidget(childWidget, event))
-            {
-                GdkWindow *window = gtk_widget_get_window(childWidget);
-                if(window && gdk_window_is_visible(window))
-                {
-                    // TODO: one could probably check here whether widget is enabled or not,
-                    // and accept if widget is disabled.
-                    if(isBlackListed(G_OBJECT(childWidget)))
-                    {
-                        usable = FALSE;
-                    }
-                    else if(GTK_IS_BUTTON(childWidget) && qtcWidgetGetState(childWidget)!=GTK_STATE_INSENSITIVE)
-                    {
-                        usable = false;
-                    }
-                    else if(gtk_widget_get_events(childWidget)&(GDK_BUTTON_PRESS_MASK|GDK_BUTTON_RELEASE_MASK))
-                    {
-                        // widget listening to press event
-                        usable = FALSE;
-                    }
-                    else if(GTK_IS_MENU_ITEM(childWidget))
-                    {
-                        // deal with menu item, GtkMenuItem only listen to
-                        // GDK_BUTTON_PRESS_MASK when state == GTK_STATE_PRELIGHT
-                        // so previous check are invalids :(
-                        usable = FALSE;
-                    }
-                    else if(GTK_IS_SCROLLED_WINDOW(childWidget) && (!inNoteBook || gtk_widget_is_focus( childWidget)))
-                    {
-                        // Scrolled do not send release events
-                        // to parents so not usable
-                        usable = FALSE;
-                    }
-                }
+                usable = false;
             }
 
             // if child is a container and event has been accepted so far, also check it, recursively
@@ -195,14 +204,17 @@ static gboolean qtcWMMoveChildrenUseEvent(GtkWidget *widget, GdkEventButton *eve
 
 static gboolean qtcWMMoveUseEvent(GtkWidget *widget, GdkEventButton *event)
 {
+    if(qtcWMMoveLastRejectedEvent && qtcWMMoveLastRejectedEvent==event)
+        return FALSE;
+
     if(!GTK_IS_CONTAINER(widget))
         return TRUE;
     
     // if widget is a notebook, accept if there is no hovered tab
     if(GTK_IS_NOTEBOOK(widget))
         return !qtcTabHasVisibleArrows(GTK_NOTEBOOK(widget)) && -1==qtcTabCurrentHoveredIndex(widget) && qtcWMMoveChildrenUseEvent(widget, event, FALSE);
-
-    return qtcWMMoveChildrenUseEvent(widget, event, FALSE);
+    else
+        return qtcWMMoveChildrenUseEvent(widget, event, FALSE);
 }
 
 static gboolean qtcWWMoveStartDelayedDrag(gpointer data)
@@ -221,6 +233,7 @@ static gboolean qtcWMMoveIsWindowDragWidget(GtkWidget *widget, GdkEventButton *e
         qtcWMMoveTimer=g_timeout_add(qtSettings.startDragTime, (GSourceFunc)qtcWWMoveStartDelayedDrag, NULL);
         return TRUE;
     }
+    qtcWMMoveLastRejectedEvent=event;
     return FALSE;
 }
 
